<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kfd_mainwindow" version="1">
<MenuBar>
  <Menu noMerge="1" name="file"><text>&amp;File</text>
    <Merge/>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>
  <Menu noMerge="1" name="edit"><text>&amp;Edit</text>
    <Merge/>
  </Menu>
  <Menu noMerge="1" name="view"><text>&amp;View</text>
   <Merge/>
  </Menu>
  <Menu noMerge="1" name="tools"><text>&amp;Tools</text>
    <Merge/>
  </Menu>
  <Menu noMerge="1" name="widgets"><text>&amp;Widgets</text>
    <Merge/>
  </Menu>
  <Menu noMerge="1" name="format"><text>&amp;Format</text>
    <Merge/>
  </Menu>
  <Merge/>
</MenuBar>
</kpartgui>