<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritatoolcrop" version="1">


<ToolBar name="transform_tools" position="top"><Text>Transformation Tools</Text>
<Separator/>
  <Action name="tool_crop"/>
</ToolBar>


</kpartgui>
