<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritacolorrange" version="1">
<MenuBar>
    <Menu name="Select"><text>Select</text>
	<Separator/>
	<Action name="colorrange"/>
	<Separator/>
    </Menu>
</MenuBar>
<ToolBar name="selection_tools" positiion="top"><Text>Selection Tools</Text>
	<Action name="tool_selectpicker"/>
</ToolBar>
</kpartgui>
