( cd opt/kde/lib ; rm -rf libkstore.so.2 )
( cd opt/kde/lib ; ln -sf libkstore.so.2.0.0 libkstore.so.2 )
( cd opt/kde/lib ; rm -rf libkstore.so )
( cd opt/kde/lib ; ln -sf libkstore.so.2.0.0 libkstore.so )
( cd opt/kde/lib ; rm -rf libkwmf.so.2 )
( cd opt/kde/lib ; ln -sf libkwmf.so.2.0.0 libkwmf.so.2 )
( cd opt/kde/lib ; rm -rf libkwmf.so )
( cd opt/kde/lib ; ln -sf libkwmf.so.2.0.0 libkwmf.so )
( cd opt/kde/lib ; rm -rf libkowmf.so.1 )
( cd opt/kde/lib ; ln -sf libkowmf.so.1.0.0 libkowmf.so.1 )
( cd opt/kde/lib ; rm -rf libkowmf.so )
( cd opt/kde/lib ; ln -sf libkowmf.so.1.0.0 libkowmf.so )
( cd opt/kde/lib ; rm -rf libkofficecore.so.2 )
( cd opt/kde/lib ; ln -sf libkofficecore.so.2.0.0 libkofficecore.so.2 )
( cd opt/kde/lib ; rm -rf libkofficecore.so )
( cd opt/kde/lib ; ln -sf libkofficecore.so.2.0.0 libkofficecore.so )
( cd opt/kde/lib ; rm -rf libkofficeui.so.2 )
( cd opt/kde/lib ; ln -sf libkofficeui.so.2.0.0 libkofficeui.so.2 )
( cd opt/kde/lib ; rm -rf libkofficeui.so )
( cd opt/kde/lib ; ln -sf libkofficeui.so.2.0.0 libkofficeui.so )
( cd opt/kde/lib ; rm -rf libkoscript.so.2 )
( cd opt/kde/lib ; ln -sf libkoscript.so.2.0.0 libkoscript.so.2 )
( cd opt/kde/lib ; rm -rf libkoscript.so )
( cd opt/kde/lib ; ln -sf libkoscript.so.2.0.0 libkoscript.so )
( cd opt/kde/lib ; rm -rf libkotext.so.2 )
( cd opt/kde/lib ; ln -sf libkotext.so.2.0.0 libkotext.so.2 )
( cd opt/kde/lib ; rm -rf libkotext.so )
( cd opt/kde/lib ; ln -sf libkotext.so.2.0.0 libkotext.so )
( cd opt/kde/lib ; rm -rf libkopainter.so.1 )
( cd opt/kde/lib ; ln -sf libkopainter.so.1.0.0 libkopainter.so.1 )
( cd opt/kde/lib ; rm -rf libkopainter.so )
( cd opt/kde/lib ; ln -sf libkopainter.so.1.0.0 libkopainter.so )
( cd opt/kde/lib ; rm -rf libkformulalib.so.3 )
( cd opt/kde/lib ; ln -sf libkformulalib.so.3.0.0 libkformulalib.so.3 )
( cd opt/kde/lib ; rm -rf libkformulalib.so )
( cd opt/kde/lib ; ln -sf libkformulalib.so.3.0.0 libkformulalib.so )
( cd opt/kde/lib ; rm -rf libkochart.so.1 )
( cd opt/kde/lib ; ln -sf libkochart.so.1.0.0 libkochart.so.1 )
( cd opt/kde/lib ; rm -rf libkochart.so )
( cd opt/kde/lib ; ln -sf libkochart.so.1.0.0 libkochart.so )
( cd opt/kde/lib ; rm -rf libkspreadcommon.so.0 )
( cd opt/kde/lib ; ln -sf libkspreadcommon.so.0.0.0 libkspreadcommon.so.0 )
( cd opt/kde/lib ; rm -rf libkspreadcommon.so )
( cd opt/kde/lib ; ln -sf libkspreadcommon.so.0.0.0 libkspreadcommon.so )
( cd opt/kde/lib ; rm -rf libkritacommon.so.1 )
( cd opt/kde/lib ; ln -sf libkritacommon.so.1.0.0 libkritacommon.so.1 )
( cd opt/kde/lib ; rm -rf libkritacommon.so )
( cd opt/kde/lib ; ln -sf libkritacommon.so.1.0.0 libkritacommon.so )
( cd opt/kde/lib ; rm -rf libkiviocommon.so.0 )
( cd opt/kde/lib ; ln -sf libkiviocommon.so.0.0.0 libkiviocommon.so.0 )
( cd opt/kde/lib ; rm -rf libkiviocommon.so )
( cd opt/kde/lib ; ln -sf libkiviocommon.so.0.0.0 libkiviocommon.so )
( cd opt/kde/lib ; rm -rf libkugarlib.so.1 )
( cd opt/kde/lib ; ln -sf libkugarlib.so.1.0.2 libkugarlib.so.1 )
( cd opt/kde/lib ; rm -rf libkugarlib.so )
( cd opt/kde/lib ; ln -sf libkugarlib.so.1.0.2 libkugarlib.so )
( cd opt/kde/lib ; rm -rf libkwmailmerge_interface.so.4 )
( cd opt/kde/lib ; ln -sf libkwmailmerge_interface.so.4.0.0 libkwmailmerge_interface.so.4 )
( cd opt/kde/lib ; rm -rf libkwmailmerge_interface.so )
( cd opt/kde/lib ; ln -sf libkwmailmerge_interface.so.4.0.0 libkwmailmerge_interface.so )
( cd opt/kde/lib ; rm -rf libkdchart.so.0 )
( cd opt/kde/lib ; ln -sf libkdchart.so.0.0.0 libkdchart.so.0 )
( cd opt/kde/lib ; rm -rf libkdchart.so )
( cd opt/kde/lib ; ln -sf libkdchart.so.0.0.0 libkdchart.so )
( cd opt/kde/lib ; rm -rf libkchartcommon.so.2 )
( cd opt/kde/lib ; ln -sf libkchartcommon.so.2.0.0 libkchartcommon.so.2 )
( cd opt/kde/lib ; rm -rf libkchartcommon.so )
( cd opt/kde/lib ; ln -sf libkchartcommon.so.2.0.0 libkchartcommon.so )
( cd opt/kde/lib ; rm -rf libkarboncommon.so.0 )
( cd opt/kde/lib ; ln -sf libkarboncommon.so.0.0.0 libkarboncommon.so.0 )
( cd opt/kde/lib ; rm -rf libkarboncommon.so )
( cd opt/kde/lib ; ln -sf libkarboncommon.so.0.0.0 libkarboncommon.so )
( cd opt/kde/lib ; rm -rf libkarbonbase.so.2 )
( cd opt/kde/lib ; ln -sf libkarbonbase.so.2.0.0 libkarbonbase.so.2 )
( cd opt/kde/lib ; rm -rf libkarbonbase.so )
( cd opt/kde/lib ; ln -sf libkarbonbase.so.2.0.0 libkarbonbase.so )
( cd opt/kde/lib ; rm -rf libkexisql3.so.3 )
( cd opt/kde/lib ; ln -sf libkexisql3.so.3.0.0 libkexisql3.so.3 )
( cd opt/kde/lib ; rm -rf libkexisql3.so )
( cd opt/kde/lib ; ln -sf libkexisql3.so.3.0.0 libkexisql3.so )
( cd opt/kde/lib ; rm -rf libkexisql2.so.2 )
( cd opt/kde/lib ; ln -sf libkexisql2.so.2.0.8 libkexisql2.so.2 )
( cd opt/kde/lib ; rm -rf libkexisql2.so )
( cd opt/kde/lib ; ln -sf libkexisql2.so.2.0.8 libkexisql2.so )
( cd opt/kde/lib ; rm -rf libkexidb.so.0 )
( cd opt/kde/lib ; ln -sf libkexidb.so.0.0.0 libkexidb.so.0 )
( cd opt/kde/lib ; rm -rf libkexidb.so )
( cd opt/kde/lib ; ln -sf libkexidb.so.0.0.0 libkexidb.so )
( cd opt/kde/lib ; rm -rf libkexidbparser.so.0 )
( cd opt/kde/lib ; ln -sf libkexidbparser.so.0.0.8 libkexidbparser.so.0 )
( cd opt/kde/lib ; rm -rf libkexidbparser.so )
( cd opt/kde/lib ; ln -sf libkexidbparser.so.0.0.8 libkexidbparser.so )
( cd opt/kde/lib ; rm -rf libkexicore.so.0 )
( cd opt/kde/lib ; ln -sf libkexicore.so.0.0.8 libkexicore.so.0 )
( cd opt/kde/lib ; rm -rf libkexicore.so )
( cd opt/kde/lib ; ln -sf libkexicore.so.0.0.8 libkexicore.so )
( cd opt/kde/lib ; rm -rf libkexiguiutils.so.0 )
( cd opt/kde/lib ; ln -sf libkexiguiutils.so.0.0.8 libkexiguiutils.so.0 )
( cd opt/kde/lib ; rm -rf libkexiguiutils.so )
( cd opt/kde/lib ; ln -sf libkexiguiutils.so.0.0.8 libkexiguiutils.so )
( cd opt/kde/lib ; rm -rf libkexidatatable.so.0 )
( cd opt/kde/lib ; ln -sf libkexidatatable.so.0.0.8 libkexidatatable.so.0 )
( cd opt/kde/lib ; rm -rf libkexidatatable.so )
( cd opt/kde/lib ; ln -sf libkexidatatable.so.0.0.8 libkexidatatable.so )
( cd opt/kde/lib ; rm -rf libkexiextendedwidgets.so.0 )
( cd opt/kde/lib ; ln -sf libkexiextendedwidgets.so.0.0.8 libkexiextendedwidgets.so.0 )
( cd opt/kde/lib ; rm -rf libkexiextendedwidgets.so )
( cd opt/kde/lib ; ln -sf libkexiextendedwidgets.so.0.0.8 libkexiextendedwidgets.so )
( cd opt/kde/lib ; rm -rf libkeximain.so.0 )
( cd opt/kde/lib ; ln -sf libkeximain.so.0.0.8 libkeximain.so.0 )
( cd opt/kde/lib ; rm -rf libkexipropertyeditor.so.0 )
( cd opt/kde/lib ; ln -sf libkexipropertyeditor.so.0.0.8 libkexipropertyeditor.so.0 )
( cd opt/kde/lib ; rm -rf libkexipropertyeditor.so )
( cd opt/kde/lib ; ln -sf libkexipropertyeditor.so.0.0.8 libkexipropertyeditor.so )
( cd opt/kde/lib ; rm -rf libkexirelationsview.so.0 )
( cd opt/kde/lib ; ln -sf libkexirelationsview.so.0.0.8 libkexirelationsview.so.0 )
( cd opt/kde/lib ; rm -rf libkexirelationsview.so )
( cd opt/kde/lib ; ln -sf libkexirelationsview.so.0.0.8 libkexirelationsview.so )
( cd opt/kde/lib ; rm -rf libkformdesigner.so.0 )
( cd opt/kde/lib ; ln -sf libkformdesigner.so.0.0.8 libkformdesigner.so.0 )
( cd opt/kde/lib ; rm -rf libkformdesigner.so )
( cd opt/kde/lib ; ln -sf libkformdesigner.so.0.0.8 libkformdesigner.so )
( cd opt/kde/lib ; rm -rf libkeximain.so )
( cd opt/kde/lib ; ln -sf libkeximain.so.0.0.8 libkeximain.so )
( cd opt/kde/lib ; rm -rf libkeximigrate.so.0 )
( cd opt/kde/lib ; ln -sf libkeximigrate.so.0.0.8 libkeximigrate.so.0 )
( cd opt/kde/lib ; rm -rf libkeximigrate.so )
( cd opt/kde/lib ; ln -sf libkeximigrate.so.0.0.8 libkeximigrate.so )
( cd opt/kde/lib ; rm -rf libkexiformutils.so.0 )
( cd opt/kde/lib ; ln -sf libkexiformutils.so.0.0.8 libkexiformutils.so.0 )
( cd opt/kde/lib ; rm -rf libkexiformutils.so )
( cd opt/kde/lib ; ln -sf libkexiformutils.so.0.0.8 libkexiformutils.so )
( cd opt/kde/lib ; rm -rf libkwordexportfilters.so.1 )
( cd opt/kde/lib ; ln -sf libkwordexportfilters.so.1.0.1 libkwordexportfilters.so.1 )
( cd opt/kde/lib ; rm -rf libkwordexportfilters.so )
( cd opt/kde/lib ; ln -sf libkwordexportfilters.so.1.0.1 libkwordexportfilters.so )
( cd opt/kde/share/apps/kivio/autoloadStencils ; rm -rf straight_connector.so )
( cd opt/kde/share/apps/kivio/autoloadStencils ; ln -sf /opt/kde/lib/kde3/straight_connector.so straight_connector.so )
( cd opt/kde/share/doc/HTML/en/koffice-apidocs ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/koffice-apidocs ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/koffice ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/koffice ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kchart ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kchart ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kformula ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kformula ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kivio ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kivio ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/koshell ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/koshell ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kpresenter ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kpresenter ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kspread ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kspread ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kugar ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kugar ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/kword ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/kword ; ln -sf /opt/kde/share/doc/HTML/en/common common )
( cd opt/kde/share/doc/HTML/en/thesaurus ; rm -rf common )
( cd opt/kde/share/doc/HTML/en/thesaurus ; ln -sf /opt/kde/share/doc/HTML/en/common common )
