<!DOCTYPE kpartgui>
<kpartgui name="kexireportpartinst" version="1">

<MenuBar>
  <Menu name="edit">
   <Action name="fompart_clear_contents"/>
  </Menu>
  <Menu name="format" noMerge="1">
  <text>&amp;Format</text>
   <Action name="snap_to_grid"/>
   <Separator/>
   <Action name="reportpart_align_menu"/>
   <Action name="reportpart_adjust_size_menu"/>
   <Separator/>
   <Action name="reportpart_format_raise"/>
   <Action name="reportpart_format_lower"/>
  </Menu>
</MenuBar>

<ToolBar name="widgets" fullWidth="false">
 <text>Widgets</text>
   <Action name="pointer"/>
   <Separator/>
   <Action name="library_widget_KexiSubReport"/>
   <Separator/>
   <Action name="library_widget_Label"/>
   <Action name="library_widget_PicLabel"/>
   <Action name="library_widget_ReportLine"/>
</ToolBar>
<ToolBar name="format" fullWidth="false" noMerge="1">
<text>Format Toolbar</text>
   <Action name="reportpart_align_menu"/>
   <Action name="reportpart_adjust_size_menu"/>
</ToolBar>

</kpartgui>

