<!DOCTYPE kpartgui>
<kpartgui name="kexiformpartinst" version="7">

<MenuBar>
  <Menu name="edit">
   <Action name="fompart_clear_contents"/>
   <Separator />
   <Action name="formpart_taborder"/>
   <Action name="formpart_adjust_size"/>
   <Action name="formpart_pixmap_collection"/>
   <Action name="formpart_connections"/>
   <!-- Action name="change_style"/ -->
  </Menu>
  <Menu name="format" noMerge="1">
  <text>&amp;Format</text>
   <Action name="snap_to_grid"/>
   <Separator/>
   <Action name="formpart_layout_menu"/>
   <Action name="formpart_break_layout"/>
   <Separator/>
   <Action name="formpart_align_menu"/>
   <Action name="formpart_adjust_size_menu"/>
   <Separator/>
   <Action name="formpart_format_raise"/>
   <Action name="formpart_format_lower"/>
  </Menu>
</MenuBar>

<ToolBar name="widgets" fullWidth="false">
 <text>Widgets</text>
   <Action name="pointer"/>
   <!-- Action name="drag_connection"/ -->
   <Separator/>
   <Action name="library_widget_KexiDBInputWidget"/>
   <Action name="library_widget_KexiLabel"/>
   <Action name="library_widget_KexiPictureLabel"/>
   <Action name="library_widget_KexiDBLineEdit"/>
   <Action name="library_widget_KIntSpinBox"/>
   <Action name="library_widget_KPushButton"/>
   <Action name="library_widget_KComboBox"/>
   <Action name="library_widget_QRadioButton"/>
   <Action name="library_widget_QCheckBox"/>
   <Action name="library_widget_Spacer"/>
   <Action name="library_widget_Line"/>
   <Separator/>
   <Action name="library_widget_QFrame"/>
   <Action name="library_widget_QGroupBox"/>
   <Action name="library_widget_KFDTabWidget"/>
   <Action name="library_widget_KexiSubForm"/>
   <Separator/>
   <Action name="library_widget_Spring"/>
   <ActionList name="library_widgets" />
</ToolBar>
<ToolBar name="format" fullWidth="false" noMerge="1">
<text>Format</text>
   <!-- Action name="formpart_layout_menu"/ -->
   <Action name="formpart_align_menu"/>
   <Action name="formpart_adjust_size_menu"/>
   <Action name="show_form_ui" />
</ToolBar>
<!-- ToolBar name="tools" fullWidth="false">
   <Action name="change_style"/>
</ToolBar -->

</kpartgui>

