<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritaexample" version="7">
<MenuBar>
    <Menu name="Filter"><text>&amp;Filter</text>
    <Seperator/>
    <Action name="krita_example"/>
    <Action name="krita_example_iterators"/>
    </Menu>
</MenuBar>
</kpartgui>
