<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritaselectiontools" version="1">

<ToolBar name="selection_tools" position="left"><Text>Selection Tools</Text>
  <Action name="tool_select_brush"/>
  <Action name="tool_select_eraser"/>
  <Action name="tool_select_freehand"/>
  <Action name="tool_select_rectangular"/>
  <Action name="tool_select_elliptical"/>
  <Action name="tool_select_polygonal"/>
  <Action name="tool_select_contiguous"/>
</ToolBar>


</kpartgui>
