/***************************************************************************
              kugar.h  -  Kugar library includes
              -------------------
    begin     : Tue Feb 29 2000                                           
    copyright : (C) 2000 by Mutiny Bay Software                         
    email     : info@mutinybaysoftware.com                                     
 ***************************************************************************/

#ifndef _KUGAR_H
#define _KUGAR_H

#include "mpagecollection.h"
#include "mreportengine.h"
#include "kreportviewer.h"
#include "mdatabasereportengine.h"

#endif
