<!DOCTYPE kpartgui>
<kpartgui name="kexitablepartinst" version="5">

<MenuBar>
  <Menu name="edit" noMerge="0">
    <text>&amp;Edit</text>
    <Separator/>
    <Action name="tablepart_toggle_pkey"/>
  </Menu>
</MenuBar>

<ToolBar name="design" fullWidth="false" noMerge="0">
  <text>Design</text>
  <!-- Design View -->
  <!-- TODO: reenable after shared toggle actions fix: Action name="tablepart_toggle_pkey"/ -->
</ToolBar>

</kpartgui>
