<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritadefaulttools" version="1">

<ToolBar name="navigation" position="top"><Text>Navigation</Text>
  <Separator/>
  <Action name="tool_move"/>
  <Action name="tool_zoom"/>
  <Action name="tool_pan"/>
</ToolBar>

<ToolBar name="painting_tools" position="left"><Text>Painting Tools</Text>
  <Action name="tool_pen"/>
  <Action name="tool_brush"/>
  <Action name="tool_airbrush"/>
  <Action name="tool_eraser"/>
  <Action name="tool_duplicate"/>
  <Action name="tool_filter"/>
  <Action name="tool_stamp"/>
  <Separator/>
  <Action name="tool_line"/>
  <Action name="tool_polyline"/>
  <Action name="tool_polygon"/>
  <Action name="tool_rectangle"/>
  <Action name="tool_ellipse"/>
  <Separator/>
  <Action name="tool_fill"/>
  <Action name="tool_gradient"/>
  <Separator/>
  <Action name="tool_text"/>
  <Separator/>
  <Action name="tool_colorchanger"/>
  <Action name="tool_colorpicker"/>
</ToolBar>

</kpartgui>
