<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kfd_part" version="2">

<MenuBar>
  <Menu name="edit" noMerge="1">
    <Action name="edit_undo" group="edit_undo_merge"/>
    <Action name="edit_redo" group="edit_undo_merge"/>
    <Separator group="edit_undo_merge"/>
    <Action name="edit_cut" group="edit_paste_merge"/>
    <Action name="edit_copy" group="edit_paste_merge"/>
    <Action name="edit_paste" group="edit_paste_merge"/>
    <Action name="edit_delete" group="edit_paste_merge"/>
    <Action name="clear_contents" group="edit_paste_merge"/>
    <Action name="edit_select_all" group="edit_select_merge"/>
  </Menu>
  <Menu name="tools" noMerge="1">
  <Merge/>
  <Separator/>
   <Action name="taborder"/>
   <Action name="change_style"/>
   <Action name="pixmap_collection"/>
   <Action name="form_connections"/>
  </Menu>
  <Menu name="widgets" noMerge="1">
   <text>&amp;Widgets</text>
   <Merge/>
   <Action name="pointer"/>
   <Action name="drag_connection"/>
   <Separator/>
   <Action name="library_widget_SubForm"/>
   <Action name="library_widget_QGroupBox"/>
   <Action name="library_widget_KFDTabWidget"/>
   <Action name="library_widget_QWidgetStack"/>
   <Action name="library_widget_QFrame"/>
   <Separator/>
   <Action name="library_widget_KLineEdit"/>
   <Action name="library_widget_QLabel"/>
   <Action name="library_widget_KexiPictureLabel"/>
   <Action name="library_widget_KPushButton"/>
   <Action name="library_widget_QRadioButton"/>
   <Action name="library_widget_QCheckBox"/>
   <Action name="library_widget_KIntSpinBox"/>
   <Action name="library_widget_KComboBox"/>
   <Action name="library_widget_KListBox"/>
   <Action name="library_widget_KTextEdit"/>
   <Action name="library_widget_KListView"/>
   <Action name="library_widget_QSlider"/>
   <Action name="library_widget_KProgress"/>
   <Action name="library_widget_KTimeWidget"/>
   <Action name="library_widget_KDateWidget"/>
   <Action name="library_widget_KDateTimeWidget"/>
   <Action name="library_widget_Line"/>
   <Action name="library_widget_Spring"/>
  </Menu>
  <Menu name="format" noMerge="1">
  <text>&amp;Format</text>
   <Action name="snap_to_grid"/>
   <Separator/>
   <Action name="layout_menu"/>
   <Action name="break_layout"/>
   <Separator/>
   <Action name="formpart_break_layout"/>
   <Action name="align_menu"/>
   <Action name="adjust_size_menu"/>
   <Separator/>
   <Action name="format_raise"/>
   <Action name="format_lower"/>
  </Menu>
</MenuBar>

<ToolBar name="fileToolBar" noMerge="1"><text>Main Toolbar</text>
    <Action name="preview_form"/>
    <Separator/>
    <Action name="edit_undo"/>
    <Action name="edit_redo"/>
    <Separator/>
    <Action name="edit_cut"/>
    <Action name="edit_copy"/>
    <Action name="edit_paste"/>
</ToolBar>
<ToolBar name="containers" fullWidth="false" noMerge="1">
 <text>Containers</text>
   <Action name="library_widget_SubForm"/>
   <Action name="library_widget_QGroupBox"/>
   <Action name="library_widget_KFDTabWidget"/>
   <Action name="library_widget_QWidgetStack"/>
   <Action name="library_widget_QFrame"/>
</ToolBar>
<ToolBar name="widgets" fullWidth="false" noMerge="1">
 <text>Widgets</text>
   <Action name="pointer"/>
   <Action name="drag_connection"/>
   <Separator/>
   <Action name="library_widget_KLineEdit"/>
   <Action name="library_widget_QLabel"/>
   <Action name="library_widget_KexiPictureLabel"/>
   <Action name="library_widget_KPushButton"/>
   <Action name="library_widget_QRadioButton"/>
   <Action name="library_widget_QCheckBox"/>
   <Action name="library_widget_KIntSpinBox"/>
   <Action name="library_widget_KComboBox"/>
   <Action name="library_widget_KListBox"/>
   <Action name="library_widget_KTextEdit"/>
   <Action name="library_widget_KListView"/>
   <Action name="library_widget_QSlider"/>
   <Action name="library_widget_KProgress"/>
   <Action name="library_widget_KTimeWidget"/>
   <Action name="library_widget_KDateWidget"/>
   <Action name="library_widget_KDateTimeWidget"/>
   <Action name="library_widget_Line"/>
   <Action name="library_widget_Spring"/>
   <ActionList name="undo_actions" />
</ToolBar>
<ToolBar name="tools" fullWidth="false" noMerge="1">
<text>Tools Toolbar</text>
   <Action name="pixmap_collection"/>
  <!-- <Action name="change_style"/> !-->
</ToolBar>
<ToolBar name="format" fullWidth="false" noMerge="1">
<text>Format Toolbar</text>
   <Action name="align_menu"/>
   <Action name="adjust_size_menu"/>
</ToolBar>

</kpartgui>