<!DOCTYPE kpartgui>
<kpartgui name="koshell" version="1">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="save_all"/>
  </Menu>
  <Menu name="settings"><Text>&amp;Settings</Text>
     <Action name="show_sidebar"/>
  </Menu>
</MenuBar>
</kpartgui>
