<!DOCTYPE kpartgui>
<kpartplugin name="insertknotsplugin" library="karbon_insertknotsplugin">
<MenuBar>
	<Menu name="plugins">
		<Action name="path_insert_knots"/>
	</Menu>
</MenuBar>
<ToolBar name="Plugins" fullWidth="false">
	<Action name="path_insert_knots"/>
</ToolBar>
</kpartplugin>
