<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd" >
<kpartgui name="KivioSelectTool" library="libkivioselecttool" version="6">
  <MenuBar>
    <Menu name="Tools">
      <Action name="select" />
    </Menu>
  </MenuBar>
  <ToolBar name="ToolsToolBar">
    <Action name="select" />
  </ToolBar>

  <Menu name="StencilPopup">
    <Action name="editText" />
    <Separator />
    <Action name="formatText" />
    <Action name="formatStencil" />
    <Action name="formatConnector" />
  </Menu>
</kpartgui>
