<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritacolorsfilters" version="8">
<MenuBar>
	<Menu name="Adjust"><text>Adjust</text>
		<Separator/>
		<Action name="brightnesscontrast"/>
		<Action name="autocontrast" />	
		<Action name="gammacorrection"/>
		<Action name="coloradjustment"/>
		<Action name="desaturate"/>	
	</Menu>
</MenuBar>
</kpartgui>
