<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritacolorspaceconversion" version="1">
<MenuBar>
    <Menu name="Image"><text>Image</text>
	    <Menu name="Mode"><text>&amp;Mode</text>
		<Action name="imgcolorspaceconversion"/>
    	    </Menu>
    </Menu>
    <Menu name="Layer"><text>Layer</text>
	    <Menu name="Mode"><text>&amp;Mode</text>
		<Action name="layercolorspaceconversion"/>
    	    </Menu>
    </Menu>
</MenuBar>
</kpartgui>
