<!DOCTYPE kpartgui>
<kpartgui name="kexiscriptpart" version="3">

<ToolBar name="tools" fullWidth="false" noMerge="0">
   <text>Tools Toolbar</text>
   <Action name="script_execute"/>
</ToolBar>

</kpartgui>
