<!DOCTYPE kpartgui>
<kpartgui name="kexiquerypart" version="2">

<MenuBar>
	<!-- (not needed - it is autogenerated!) <Menu name="widgets">
		<text>&amp;Create</text>
		<Action name="querypart_create"/>
	</Menu -->
</MenuBar>

</kpartgui>
