<!DOCTYPE kpartgui>
<kpartgui name="kexirelationsview" version="1">

<MenuBar>
	<Menu name="edit">
		<Action name="relationsview_openSelectedTableQuery"/>
		<Action name="relationsview_removeSelectedTableQuery"/>
		<Action name="relationsview_removeSelectedConnection"/>
	</Menu>
</MenuBar>

</kpartgui>
