<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd" ><kpartgui name="Krita" version="9">

<MenuBar>
 <Menu name="Edit"><text>&amp;Edit</text>
  <Action name="koffice_undo"/>
  <Action name="koffice_redo"/>
  <Separator/>
  <Action name="cut"/>
  <Action name="copy"/>
  <Action name="copy_merged"/>
  <Action name="paste"/>
  <Action name="clear"/>
  <Separator/>
  <Action name="revert"/>
 </Menu>

 <Menu name="View"><text>&amp;View</text>
  <Action name="fullscreen"/>
  <Separator/>
  <Action name="view_newview"/>
  <ActionList name="view_closeallviews"/>
  <Separator/>
  <ActionList name="view_split"/>
  <Action name="zoom_in"/>
  <Action name="zoom_out"/>
  <Separator/>
  <Action name="view_color_docker"/>
  <Action name="view_control_docker"/>
  <Action name="view_layer_docker"/>
  <Action name="view_shapes_docker"/>
  <Action name="view_fills_docker"/>
  <Action name="view_paintop_docker"/>
  <Separator/>
  <Action name="view_ruler"/>
  <Separator/>
  <Action name="refresh_canvas"/>
 </Menu>


<Menu name="Image"><text>&amp;Image</text>
    <Menu name="Mode"><text>&amp;Mode</text>
	<Action name="img_properties"/>
    </Menu>

  <Separator/>
  <Action name="import_image"/>
  <Action name="export_image"/>
  <Separator/>
  <Action name="add_new_image_tab"/>
  <Action name="remove_current_image_tab"/>
  <Action name="scale_image"/>
  <Action name="resizeimgtolayer"/>
  <Action name="flatten_image"/>
  <Action name="duplicate_image"/>
</Menu>

<Menu name="Adjust"><text>&amp;Adjust</text>
	<ActionList name="adjust"/>
</Menu>


<Menu name="Layer"><text>La&amp;yer</text>
  <Action name="insert_layer"/>
  <Action name="remove_layer"/>
  <Action name="duplicate_layer"/>
  <Action name="hide_layer"/>
  <Action name="link_layer"/>
  <Separator/>
  <Action name="raiselayer"/>
  <Action name="lowerlayer"/>
  <Action name="toplayer"/>
  <Action name="bottomlayer"/>
  <Separator/>
  <Action name="resizelayertoowner"/>
  <Separator/>
  <Action name="insert_image_as_layer"/>
  <Action name="save_layer_as_image"/>
  <Action name="layer_to_image"/>
<Separator/>
  <Action name="mirrorLayerX"/>
  <Action name="mirrorLayerY"/>
  <Separator/>
  <Action name="layer_properties"/>
  <Menu name="Mode"><text>&amp;Mode</text>
  </Menu>
  <Separator/>
  <Action name="merge_layer"/>
  <Action name="merge_visible_layers"/>
  <Action name="merge_linked_layers"/>
</Menu>


<Menu name="Select"><text>Se&amp;lect</text>
    <Action name="select_all"/>
    <Action name="deselect"/>
    <Action name="reselect"/>
    <Action name="copy_selection_to_new_layer"/>
    <Action name="invert"/>
    <Separator/>
    <Action name="feather"/>
    <Menu name="Modify"><text>&amp;Modify</text>
            <Action name="border"/>
            <Action name="smooth"/>
            <Action name="expand"/>
            <Action name="contract"/>
   </Menu>
   <Separator/>
   <Action name = "grow"/>
   <Action name = "similar"/>
   <Separator/>
   <Action name="transform_selection"/>
   <Separator/>
   <Action name="load_selection"/>
   <Action name="save_selection"/>
</Menu>

<Menu name="Filter"><text>F&amp;ilter</text>
	<ActionList name="filters"/>
</Menu>

<Menu name="settings"><text>Settings</text>
  <Separator group="settings_show"/>
  <Action name="preferences" group="settings_configure"/>
</Menu>

</MenuBar>

<ToolBar name="edit_toolbar" position="top"><Text>Edit</Text>
  <Action name="koffice_undo"/>
  <Action name="koffice_redo"/>
  <Separator/>
  <Action name="cut"/>
  <Action name="copy"/>
  <Action name="remove"/>
  <Action name="scrap_selection"/>
  <Action name="paste_special"/>
</ToolBar>

<ToolBar name="navigation" position="top"><Text>Navigation</Text>
  <Action name="refresh_canvas"/>
  <Action name="zoom_in"/>
  <Action name="zoom_out"/>
</ToolBar>


<ToolBar name="painting_tools" position="left"><Text>Painting Tools</Text>
</ToolBar>

<ToolBar name="transform_tools" position="top"><Text>Transformation Tools</Text>
	<Action name="mirrorLayerX"/>
	<Action name="mirrorLayerY"/>
</ToolBar>

<ToolBar name="selection_tools" position="left"><Text>Selection Tools</Text>
</ToolBar>



<Menu name="menuimage_popup">
  <Action name="rename_current_image_tab"/>
  <Action name="add_new_image_tab"/>
  <Action name="remove_current_image_tab"/>
</Menu>

</kpartgui>
