<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kexi" version="129">
<MenuBar>
  <Menu name="file" noMerge="0">
    <text>&amp;Project</text>
    <Action name="project_new" />
    <Action name="project_open"/>
    <Action name="project_download_examples"/>
    <Action name="project_open_recent"/>
    <!-- Menu name="project_open_recent">
      <text>Open Recent</text>
      <MergeLocal name="recent_actions"/>
      <Separator />
      <Action name="project_open_recent_more"/>
    </Menu -->
    <Separator />
    <Action name="project_save"/>
    <Action name="project_saveas"/>
    <!-- Separator />
    <Menu name="project_import">
      <text>&amp;Import</text>
      <Action name="project_import_project" />
      <Action name="project_import_file" />
      <Action name="project_import_server" />
    </Menu -->
    <!-- Menu name="project_export">
      <text>&amp;Export</text>
    </Menu -->
    <Separator />
    <Action name="project_relations"/>
    <Action name="project_properties"/>
    <Separator />
    <Action name="project_close"/>
    <Separator />
    <Action name="quit"/>
  </Menu>

  <Menu name="edit" noMerge="1">
    <text>&amp;Edit</text>
    <Action name="edit_undo"/>
    <Action name="edit_redo"/>
    <MergeLocal name="edit_undo_merge"/>
    <Separator/>
    <Action name="edit_cut"/>
    <Action name="edit_copy"/>
    <Action name="edit_paste"/>
    <MergeLocal name="edit_paste_merge"/>
    <Separator/>
    <Action name="edit_select_all"/> 
    <Separator/>
    <Action name="edit_delete"/>
    <Action name="edit_delete_row"/>
    <ActionList name="edit_clear_table"/>
    <ActionList name="edit_insert_empty_row"/>
    <DefineGroup name="edit_find"/>
    <Merge/>
  </Menu>
  
  <Menu name="view" noMerge="1">
    <text>&amp;View</text>
    <Action name="view_data_mode" type="toggle" />
    <Action name="view_design_mode" type="toggle" />
    <Action name="view_text_mode" type="toggle" />
    <Separator/>
    <Action name="view_navigator"/>
    <Action name="view_propeditor"/>
    <Separator/>
    <MergeLocal/>
  </Menu>

  <Menu name="create" noMerge="1">
    <text>&amp;Create</text>
  </Menu>

  <Menu name="data" noMerge="1">
    <text>&amp;Data</text>
    <ActionList name="data_save_row"/>
    <ActionList name="data_cancel_row_changes"/>
    <ActionList name="data_filter"/>
    <Menu name="data_sort">
      <text>Sort</text>
      <Action name="data_sort_az"/>
      <Action name="data_sort_za"/>
    </Menu>
  </Menu>

  <Merge/>

  <Menu name="tools" noMerge="0">
    <text>&amp;Tools</text>
    <Menu name="tools_migration">
      <text>&amp;Migration</text>
      <Action name="tools_import_project"/>
    </Menu>
    <MergeLocal/>
  </Menu>

  <Menu name="window" noMerge="0">
    <text>&amp;Window</text>
    <MergeLocal/>
  </Menu>
  
  <Menu name="settings" noMerge="1">
    <text>&amp;Settings</text>
    <Action name="options_show_menubar"/>
    <Action name="options_show_toolbar"/>
    <Merge name="StandardToolBarMenuHandler"/>
    <Merge name="KMDIViewActions"/>
    <MergeLocal name="show_toolbar_merge"/>
    <Action name="options_show_statusbar"/>
    <MergeLocal name="show_merge"/>
    <Separator/>
    <!-- Menu name="options_show_other">
      <Text>&amp;Other</Text>
      <Action name="options_show_nav"/>
      <Action name="options_show_contexthelp"/>
      <Action name="options_enable_forms"/>
    </Menu -->
    <Separator/>
    <Action name="options_configure_keybinding"/>
    <Action name="options_configure_toolbars"/>
    <Action name="options_configure"/>
  </Menu>
  
  <Menu name="help">
    <text>&amp;Help</text>
    <Action name="help_show_important_info" noMerge="0"/>
    <!-- not yet needed: Menu name="help_licenses" noMerge="0">
    <text>Other &amp;Licenses</text>
      <Merge />
    </Menu -->
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar" noMerge="1" fullWidth="false">
  <text>Main Toolbar</text>
  <Action name="project_save"/>
  <Separator/>
  <Action name="edit_undo"/>
  <Action name="edit_redo"/>
  <Action name="edit_cut"/>
  <Action name="edit_copy"/>
  <Action name="edit_paste"/>
  <!-- Separator/>
  <Action name="edit_find"/>
  <Action name="view_zoom"/>
  <Separator/ -->
  <Action name="project_relations"/>
</ToolBar>

<ToolBar name="viewToolBar" fullWidth="false" noMerge="1">
  <text>View</text>
  <Action name="view_data_mode" type="toggle" />
  <Action name="view_design_mode" type="toggle" />
  <Action name="view_text_mode" type="toggle" />
</ToolBar>

<ToolBar name="data" fullWidth="false" noMerge="0">
  <text>Data</text>
  <ActionList name="data_cancel_row_changes"/>
  <ActionList name="data_save_row"/>
  <ActionList name="data_sort_az"/>
  <ActionList name="data_sort_za"/>
  <ActionList name="data_filter"/>
</ToolBar>

</kpartgui>
