<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritaoilpaintfilter" version="4">
  <MenuBar>
    <Menu name="Filter"><text>&amp;Filter</text>
      <Action name="oilpaint_filter"/>
    </Menu>
  </MenuBar>
</kpartgui>
