<!DOCTYPE kpartgui>
<kpartplugin name="whirlpinchplugin" library="karbon_whirlpinchplugin">
<MenuBar>
	<Menu name="plugins">
		<Action name="path_whirlpinch"/>
	</Menu>
</MenuBar>
<ToolBar name="Plugins" hidden="true" fullWidth="false">
	<Action name="path_whirlpinch"/>
</ToolBar>
</kpartplugin>
