<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd" ><kpartgui name="Krita" version="1">

<MenuBar>
 <Menu name="edit"><text>Edit</text>
  <Separator/>
  <Action name="copy"/>
  <Separator/>
  <Action name="select_all"/>
  <Action name="select_none"/>
  <Action name="flip_selection"/>  
 </Menu>

 <Menu name="view"><text>View</text>
  <Action name="zoom_in"/>
  <Action name="zoom_out"/>
 </Menu>

<Menu name="Layer"><text>Layer</text>
  <Action name="hide_layer"/>
  <Action name="previous_layer"/>  
  <Action name="next_layer"/>
</Menu>

</MenuBar>

<ToolBar name="edit" position="top"><Text>Edit</Text>
  <Action name="copy"/>
</ToolBar>

<ToolBar name="navigation" position="top"><Text>Navigation</Text>
  <Action name="zoom_in"/>
  <Action name="zoom_out"/>
</ToolBar>

</kpartgui>
