/*
 * This file is part of the KDE project
 * Copyright (C) 2000-2001 theKompany.com & Dave Marotti
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef KO_TOOLDOCKMANAGER_H
#define KO_TOOLDOCKMANAGER_H

#include <qobject.h>
#include <qwidget.h>
#include <qptrlist.h>
#include <qtimer.h>
#include <koffice_export.h>
class KoToolDockBase;
class KoTabbedToolDock;

/**
 * Manager class for a collection of floating palette-like dockers.
 */
class KOFFICEUI_EXPORT KoToolDockManager : public QObject
{

    Q_OBJECT

public:
    KoToolDockManager( QWidget* parent, const char* name = 0 );

    ~KoToolDockManager();

    bool eventFilter( QObject*, QEvent* );

    KoToolDockBase* createSimpleToolDock( QWidget* view, const char* name = 0 );
    KoTabbedToolDock* createTabbedToolDock(const char* name = 0);

protected:

    void addToolWindow( KoToolDockBase* );

protected slots:

    void removeToolWindow();
    void slotTimeOut();
    void removeToolWindow( KoToolDockBase* );

private:
    QTimer timer;
    QWidget* m_pView;
    QPtrList<KoToolDockBase> tools;
};

#endif // KO_TOOLDOCKMANAGER_H

