
<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritaimagesize" version="6">
<MenuBar>
    <Menu name="Image"><text>Image</text>
	<Separator/>
	<Action name="imagesize"/>
    </Menu>
    <Menu name="Layer"><text>Layer</text>
	<Separator/>
	<Action name="layersize"/>
    </Menu>
    <Menu name="Adjust"><text>&amp;Adjust</text>
	<Separator/>
	<Action name="selectionscale"/>
    </Menu>
</MenuBar>
</kpartgui>
