<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui library="kritaembossfilter" version="4">
  <MenuBar>
    <Menu name="Filter"><text>&amp;Filter</text>
      	<Menu name="Emboss"><text>&amp;Emboss</text>
   	   <Action name="emboss_filter"/>
	</Menu>
    </Menu>
  </MenuBar>
</kpartgui>
